function D = odctndict(n,L,p)
%ODCTNDICT Multi-dimensional overcomplete DCT dictionary.
%  D = ODCTNDICT([N1 N2 ... Np],[L1 L2 ... Lp]) returns an overcomplete DCT
%  dictionary for p-dimensional signals of size N1xN2x...xNp. The number of
%  dictionary atoms used to represent the i-th signal dimension is Li, so
%  the combined dictionary has L1*L2*...*Lp atoms.
%
%  D = ODCTNDICT([N1 N2 ... Np],L) can be used to specify L - the total
%  number of atoms required in the dictionary, instead of the each of the
%  Li's individually. This call sets the Li's automatically so that their
%  relative sizes are roughly proportional to the Ni's. The actual number
%  of atoms in the returned dictionary is equal to or larger than L, as
%  integer rounding may be required to determine the Li's.
%
%  D = ODCTNDICT(N,L,P) is shorthand to the call ODCTNDICT(N*ones(1,p),L)
%  and returns the overcomplete DCT dictionary for P-dimensional signals of
%  size NxNx...xN. L is the required size of the overcomplete dictionary,
%  and is rounded up to the nearest integer whose P-th root is whole.
%
%  See also ODCTDICT, ODCT2DICT, ODCT3DICT.

%  Ron Rubinstein
%  Computer Science Department
%  Technion, Haifa 32000 Israel
%  ronrubin@cs
%
%  February 2008

if (nargin==2)
  p = length(n);
end

if (length(n)==1)
  n = n*ones(1,p);
end

if (length(L)==1 && p>1)
  N = prod(n);
  L = ceil((L*(n.^p/N).^(1/(p-1))).^(1/p));
end

D = odctdict(n(1),L(1));
for i = 2:p
  D = kron(D,odctdict(n(i),L(i)));
end
